package com.streambase.contrib.rssreader;

import java.beans.*;

import com.streambase.sb.operator.parameter.*;

/**
 * A BeanInfo class controls what properties are exposed, add 
 * metadata about properties (such as which properties are optional), and access 
 * special types of properties that can't be automatically derived via reflection. 
 * If a BeanInfo class is present, only the properties explicitly declared in
 * this class will be exposed by StreamBase.
 */
public class DynamicRSSReaderBeanInfo extends SimpleBeanInfo {
    public static final String RSS_FEEDS_PROPERTY_NAME = "RSSFeeds";
    public static final String PERIOD_PROPERTY_NAME = "Period";
    public static final String NUM_RETRIES_PROPERTY_NAME = "NumRetries";
    public static final String TITLE_PROPERTY_NAME = "Title";
    public static final String LINK_PROPERTY_NAME = "Link";
    public static final String URI_PROPERTY_NAME = "Uri";
    public static final String DESCRIPTION_PROPERTY_NAME = "Descr";
    public static final String CONTENTS_PROPERTY_NAME = "Contents";
    public static final String AUTHOR_PROPERTY_NAME = "Author";
    public static final String CATEGORIES_PROPERTY_NAME = "Cat";
    public static final String PUBDATE_AS_STRING_PROPERTY_NAME = "Pubdate";
    public static final String PUBDATE_AS_TIMESTAMP_PROPERTY_NAME = "PubdateAsTimestamp";
	public static final String CONTROL_PORT = "Start Control Port";
	/*
	 * The order of properties below determines the order they are displayed within
	 * the StreamBase Studio property view. 
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		try {
			PropertyDescriptor[] p = {
					new SBPropertyDescriptor("rssFeeds", DynamicRSSReader.class)
							.displayName("RSS Feeds").description("List of your RSS/Atom feeds to monitor. Unique key identifiers must be specified in both the dynamic mode and the static mode. When there are duplicate keys, then the first feed with that name will be monitored.").optional(),
					new SBPropertyDescriptor("pollFrequency",
							DynamicRSSReader.class).displayName(
							"Poll Frequency").description("The time, in seconds, to wait between looking for new feed entries"),
					new SBPropertyDescriptor("connectionRetries",
							DynamicRSSReader.class).displayName(
							"Number of Connection Retries").description("Number of times the adapter should try to reconnect when timing out while accessing a feed (-1 to try forever)"),					
					new SBPropertyDescriptor("title", DynamicRSSReader.class)
							.displayName("Show Title").description("Show the title of the item"),
					new SBPropertyDescriptor("link", DynamicRSSReader.class)
							.displayName("Show Link").description("Show the URL of the item"),
					new SBPropertyDescriptor("uri", DynamicRSSReader.class)
							.displayName("Show URI").description("Shows the HTTP web address of the item"),
					new SBPropertyDescriptor("description",
							DynamicRSSReader.class).displayName(
							"Show Description").description("Show the item synopsis (RSS)"),
					new SBPropertyDescriptor("contents", DynamicRSSReader.class)
							.displayName("Show Contents").description("Show the item synopsis (Atom)"),
					new SBPropertyDescriptor("author", DynamicRSSReader.class)
							.displayName("Show Author").description("Show the email address of the author of the item"),
					new SBPropertyDescriptor("categories",
							DynamicRSSReader.class).displayName(
							"Show Categories").description("Show the categories the item is included in"),
					new SBPropertyDescriptor("publishedDateAsString",
							DynamicRSSReader.class).displayName(
							"Show Published Date (As String)").description("Show the date when the item was published (as a String)"),
					new SBPropertyDescriptor("PublishedDateAsTimestamp",
							DynamicRSSReader.class).displayName(
							"Show Published Date (As Timestamp)").description("Show the date when the item was published (as a Timestamp)")};
				return p;
		} catch (IntrospectionException e) {
			System.err.println("Failed to create property descriptors");
			e.printStackTrace();
			return null;
		}
	}

}
